@@ -1,5 +1,4 @@
 import re
-import xml.sax
 from typing import Optional, Any, Union, List, Dict, Tuple
 from typing.io import IO
 from xml.etree.ElementTree import parse, Element
@@ -32,17 +31,20 @@ def extract_namespace_name(value: str) -> Tuple[str, str]:
 class XmlNode(AbstractNode):
 
     def __init__(self, element: Element, parent: DrbNode = None, **kwargs):
-        super().__init__()
+        AbstractNode.__init__(self)
         namespace_uri, name = extract_namespace_name(element.tag)
         self._name = name
         self._namespace_uri = namespace_uri
+        if self._namespace_uri is None:
+            self._namespace_uri = element.get('xmlns', None)
+        if self._namespace_uri is None and parent is not None:
+            self._namespace_uri = parent.namespace_uri
+
         self._parent = parent
         self._attributes = None
         self._children = None
         self._elem: Element = element
-        self._path = None
-        if 'path' in kwargs.keys() and isinstance(kwargs['path'], ParsedPath):
-            self._path = kwargs['path'] / name
+        self._occurrence = kwargs.get('occurrence', 1)
 
     @property
     def name(self) -> str:
@@ -60,12 +62,9 @@ class XmlNode(AbstractNode):
 
     @property
     def path(self) -> ParsedPath:
-        if self._path is None:
-            if self._parent is None:
-                self._path = ParsedPath(f'/{self._name}')
-            else:
-                self._path = self.parent.path / self.name
-        return self._path
+        if self._parent is None:
+            return ParsedPath(f'/{self._name}')
+        return self.parent.path / self.name
 
     @property
     def parent(self) -> Optional[DrbNode]:
@@ -77,39 +76,41 @@ class XmlNode(AbstractNode):
             self._attributes = {}
             for k, v in self._elem.attrib.items():
                 ns, name = extract_namespace_name(k)
-                self._attributes[(name, ns)] = v
+                if name != 'xmlns' or ns is not None:
+                    self._attributes[(name, ns)] = v
         return self._attributes
 
     @property
     @drb.resolve_children
     def children(self) -> List[DrbNode]:
         if self._children is None:
-            self._children = [XmlNode(e, self) for e in list(self._elem)]
+            self._children = []
+            occurrences = {}
+            for elem in self._elem:
+                namespace, name = extract_namespace_name(elem.tag)
+                occurrence = occurrences.get(name, 0) + 1
+                occurrences[name] = occurrence
+                self._children.append(
+                    XmlNode(elem, self, occurrence=occurrence))
         return self._children
 
     def _get_named_child(self, name: str, namespace_uri: str = None,
                          occurrence: Union[int, slice] = 0) -> \
             Union[DrbNode, List[DrbNode]]:
         tag = f'ns:{name}'
+        named_children = self._elem.findall(tag, {'ns': '*'})
 
-        if namespace_uri is None:
-            if not self.namespace_aware:
-                ns = {'ns': "*"}
-            else:
-                tag = name
-                ns = {}
-        else:
-            ns = {'ns': namespace_uri}
+        if len(named_children) == 0:
+            raise DrbException(f'No child found having name: {name} and'
+                               f' namespace: {namespace_uri}')
 
-        found = self._elem.findall(tag, ns)
+        children = [XmlNode(named_children[i], self, occurrence=i+1)
+                    for i in range(len(named_children))]
+        if self.namespace_aware or namespace_uri is not None:
+            children = list(
+                filter(lambda n: n.namespace_uri == namespace_uri, children))
 
-        if len(found) > 0:
-            if isinstance(occurrence, slice):
-                return [XmlNode(e, parent=self) for e in found][occurrence]
-            else:
-                return XmlNode(found[occurrence], parent=self)
-        raise DrbException(f'No child found having name: {name} and'
-                           f' namespace: {namespace_uri}')
+        return children[occurrence]
 
     def has_impl(self, impl: type) -> bool:
         return impl == str and not self.has_child()
@@ -151,12 +152,17 @@ class XmlNode(AbstractNode):
         else:
             return False
 
+    def __hash__(self):
+        return hash(self.path.name) + hash(self._occurrence)
+
 
 class XmlSaxNode(DrbLogicalNode):
     def __init__(self, node: DrbNode, occurrence: int = 1):
-        super(XmlSaxNode, self).__init__(node.name,
-                                         parent=node.parent,
-                                         namespace_uri=node.namespace_uri)
+        DrbLogicalNode.__init__(self, node.name,
+                                parent=node.parent,
+                                namespace_uri=node.namespace_uri)
+        for key, value in node.attributes.items():
+            self.add_attribute(key[0], value, key[1])
         self.__occurrence = occurrence
 
     def __hash__(self):
@@ -311,8 +317,11 @@ class SaxNodeHandler(ContentHandler):
 
     def characters(self, content):
         text = ''.join(content.split())
-        if text != '':
-            self.__current.value = content
+        if text != '' and self.__current is not None:
+            if self.__current.value is None:
+                self.__current.value = content
+            else:
+                self.__current.value += content
 
     def get_node(self):
         return self.__root
@@ -331,11 +340,7 @@ class XmlBaseNode(AbstractNode):
         super().__init__()
         self.__base_node = node
         self.__source = source
-        handler = SaxNodeHandler()
-        xml.sax.parse(source, handler)
-        source.close()
-        self.__xml_node = handler.get_node()
-        self.__xml_node.parent = self.__base_node
+        self.__xml_node = XmlNode(parse(source).getroot(), node)
 
     @property
     def name(self) -> str:
