@@ -1,12 +1,11 @@
 import os
-from io import BufferedIOBase
-from xml.sax import parse, parseString
+from io import BufferedIOBase, StringIO
 from typing import Union
 from drb import DrbNode
+from drb.utils.logical_node import DrbLogicalNode
 from drb.factory.factory import DrbFactory
 from drb.exceptions import DrbFactoryException
-
-from .xml_node import XmlBaseNode, SaxNodeHandler
+from .xml_node import XmlBaseNode
 
 
 class XmlNodeFactory(DrbFactory):
@@ -16,12 +15,10 @@ class XmlNodeFactory(DrbFactory):
 
     def create(self, source: Union[DrbNode, str]) -> DrbNode:
         if isinstance(source, str):
-            handler = SaxNodeHandler()
+            base_node = DrbLogicalNode("/")
             if os.path.exists(source):
-                parse(source, handler)
-            else:
-                parseString(source, handler)
-            return handler.get_node()
+                return XmlBaseNode(base_node, source)[0]
+            return XmlBaseNode(base_node, StringIO(source))[0]
         elif isinstance(source, DrbNode):
             return self._create(source)
         raise DrbFactoryException(f'Invalid parameter type: {type(source)}')
